#include "mozilla/SSE.h"

#ifdef MOZ_SAMPLE_TYPE_FLOAT32
#define SOUNDTOUCH_FLOAT_SAMPLES 1
#else
#define SOUNDTOUCH_INTEGER_SAMPLES 1
#endif

#ifndef MOZILLA_PRESUME_SSE
#ifdef MOZ_SAMPLE_TYPE_FLOAT32
#define SOUNDTOUCH_DISABLE_X86_OPTIMIZATIONS 1
#endif
#endif

#ifndef MOZILLA_PRESUME_MMX
#ifdef MOZ_SAMPLE_TYPE_S16
#define SOUNDTOUCH_DISABLE_X86_OPTIMIZATIONS 1
#endif
#endif
